#include <Wire.h>
#include <max6675.h>
#include <EEPROM.h>
#include <PID_v1.h>
#include <UTFT.h>

UTFT myGLCD(HX8357C, 38, 39, 40, 41);

extern uint8_t SmallFont[];
extern uint8_t BigFont[];
extern uint8_t BigFontRus[]; //Кирилица
extern uint8_t SevenSegNumFont[];

int er1 = 1;
int er2 = 1;
int reg1;
int reg2;
int pwr1;
int pwr2;

boolean out1;
boolean out2;

int min_pwr_BOTTOM;              //минимальная мощность нижнего нагревателя
int max_pwr_BOTTOM;              //максимальная мощность нижнего нагревателя
int min_pwr_TOPStep[3];       //минимальная мощность верхнего нагревателя
int max_pwr_TOPStep[3];       //максимальная мощность верхнего нагревателя

//RelayPin "1"-ВЕРХНИЙ нагреватель
//RelayPin "2"-НИЖНИЙ нагреватель
#define RelayPin1 6  //назначаем пин "ВЕРХНЕГО" нагревателя
#define RelayPin2 7  //назначаем пин "НИЖНЕГО" нагревателя

// Выходы реле
#define P1_PIN 9  //назначаем пин реле 1
#define P2_PIN 10  //назначаем пин реле 2
#define P3_PIN 11  //назначаем пин реле 3
#define P4_PIN 12  //назначаем пин реле 4

int tableSize;
int buzzerPin = 8;
int time = 0;

//Назначаем пины кнопок управления
int upSwitchPin = 18;
int downSwitchPin = 19;
int cancelSwitchPin = 20;
int okSwitchPin = 21;

//состояние кнопок по умолчанию
int upSwitchState = 0;
int downSwitchState = 0;
int cancelSwitchState = 0;
int okSwitchState = 0;

//переменные для кнопок
long ms_button = 0;
boolean  button_state = false;
boolean  button_long_state = false;

//profile stuff
byte currentProfile = 1;
int currentStep = 1;
byte profileSteps;

//profile name
int profileName;

//current editing step pointer
int editStep = 0;

int dwellTimerStep[3];
double rampRateStep[3];
double TimerStep;

int kp1;
int ki1;
int kd1;
int kp2;
int ki2;
int kd2;

double startTemp;

int setpointRamp;
boolean TopStart = false;
int flag = 0;  //флаг для фиксации стартовой температуры
int x=1;  //переменная для перехода на нужный шаг при горячей плате
int temperatureStep[3];
int eepromAddress = 0;//начало eepromaddress

long previousMillis; //это для счетчиков

double counter;

//these are the different states of the sketch. We call different ones depending on conditions
// ***** TYPE DEFINITIONS *****
typedef enum REFLOW_STATE
{
  REFLOW_STATE_IDLE,
  REFLOW_STATE_MENU_STEPS,
  REFLOW_STATE_MENU_TABLE_SIZE,
  REFLOW_STATE_MENU_BOTTOM_HEAT,
  REFLOW_STATE_MENU_BOTTOM_PWR_MIN,
  REFLOW_STATE_MENU_BOTTOM_PWR_MAX,
  REFLOW_STATE_MENU_STEP_RAMP,
  REFLOW_STATE_MENU_STEP_TARGET,
  REFLOW_STATE_MENU_TOP_PWR_MIN,
  REFLOW_STATE_MENU_TOP_PWR_MAX,
  REFLOW_STATE_MENU_STEP_DWELL,
  REFLOW_STATE_MENU_BOTTOM_P,
  REFLOW_STATE_MENU_BOTTOM_I,
  REFLOW_STATE_MENU_BOTTOM_D,
  REFLOW_STATE_MENU_TOP_P,
  REFLOW_STATE_MENU_TOP_I,
  REFLOW_STATE_MENU_TOP_D,
  REFLOW_STATE_STEP_RAMP,
  REFLOW_STATE_TABLE_SIZE,
  REFLOW_STATE_STEP,
  REFLOW_STATE_STEP_DWELL,
  REFLOW_STATE_COMPLETE,
  REFLOW_STATE_ERROR
}
reflowState_t;

typedef enum REFLOW_STATUS //this is simply to check if reflow should be running or not
{
  REFLOW_STATUS_OFF,
  REFLOW_STATUS_ON
}
reflowStatus_t;

#define SENSOR_SAMPLING_TIME 1000 //read tc every second
reflowStatus_t reflowStatus;
// Reflow oven controller state machine state variable
reflowState_t reflowState;

//TC read timer variables
//unsigned long nextCheck1;
unsigned long nextRead1; //переменная для обновления текущей температуры
//PID stuff

double Setpoint1, Input1, Output1;
//Specify the links and initial tuning parameters
PID myPID1(&Input1, &Output1, &Setpoint1, kp1, ki1, kd1, DIRECT);
unsigned long windowStartTime;
//PID stuff
double Setpoint2, Input2, Output2;
//Specify the links and initial tuning parameters
PID myPID2(&Input2, &Output2, &Setpoint2, kp2, ki2, kd2, DIRECT);

//Alarm state boolean
boolean alarmOn = false;

//Update whole screen boolean
boolean updateScreen = true;

//назначаем пины уселителя термопары MAX6675 "ВЕРХНЕГО" нагревателя   clk=sck cs=cs do=so
int thermoCLK = 48;  //=sck
int thermoCS = 47;   //=cs
int thermoDO = 46;   //=so
//назначаем пины уселителя термопары MAX6675 "НИЖНЕГО" нагревателя clk=sck cs=cs do=so
int thermoCLK2 = 44;  //=sck
int thermoCS2 = 43;   //=cs
int thermoDO2 = 42;   //=so

// переменные для калмана "ВЕРХНЕГО" нагревателя
float varTerm1 = 0.25;  // среднее отклонение (ищем в excel)
float varProcess1 = 0.0125; // скорость реакции на изменение (подбирается вручную)
float Pc1 = 0.0;
float G1 = 0.0;
float P1 = 1.0;
float Xp1 = 0.0;
float Zp1 = 0.0;
float Xe1 = 0.0;

// переменные для калмана "НИЖНЕГО" нагревателя
float varTerm2 = 0.25;  // среднее отклонение (ищем в excel)
float varProcess2 = 0.0125; // скорость реакции на изменение (подбирается вручную)
float Pc2 = 0.0;
float G2 = 0.0;
float P2 = 1.0;
float Xp2 = 0.0;
float Zp2 = 0.0;
float Xe2 = 0.0;
// переменные для калмана
int tc1;
int tc2;

MAX6675 thermocouple1(thermoCLK, thermoCS, thermoDO);//термопара "ВЕРХНЕГО" нагревателя
MAX6675 thermocouple2(thermoCLK2, thermoCS2, thermoDO2);//термопара "НИЖНЕГО" нагревателя

void loadProfile()//this function loads whichever profile currentProfile variable is set to
{
  profileSteps = EEPROM.read((currentProfile - 1) * 29);
  Setpoint2 = EEPROM.read((currentProfile - 1) * 29 + 1);
  for (int i = 0; i < 3; i + 1) {
    rampRateStep[i] = EEPROM.read((currentProfile - 1) * 29 + i + 2);
    i++;
  }
  for (int i = 0; i < 3; i + 1) {
    dwellTimerStep[i] = EEPROM.read((currentProfile - 1) * 29 + i + 5) * 5;
    i++;
  }
  for (int i = 0; i < 3; i + 1) {
    temperatureStep[i] = EEPROM.read((currentProfile - 1) * 29 + i + 8);
    i++;
  }
  for (int i = 0; i < 3; i + 1) {
    min_pwr_TOPStep[i] = EEPROM.read((currentProfile - 1) * 29 + i + 11);
    i++;
  }
  for (int i = 0; i < 3; i + 1) {
    max_pwr_TOPStep[i] = EEPROM.read((currentProfile - 1) * 29 + i + 14);
    i++;
  }
  kp1 = EEPROM.read((currentProfile - 1) * 6 + 172);
  ki1 = EEPROM.read((currentProfile - 1) * 6 + 173);
  kd1 = EEPROM.read((currentProfile - 1) * 6 + 174);
  kp2 = EEPROM.read((currentProfile - 1) * 6 + 175);
  ki2 = EEPROM.read((currentProfile - 1) * 6 + 176);
  kd2 = EEPROM.read((currentProfile - 1) * 6 + 177);
  tableSize = EEPROM.read((currentProfile - 1) * 6 + 202);
  min_pwr_BOTTOM = EEPROM.read((currentProfile - 1) * 6 + 203);
  max_pwr_BOTTOM = EEPROM.read((currentProfile - 1) * 6 + 204);

  return;
}


void setup()
{
  Serial.begin(9600);
  
//setup pins as input for buttons
  pinMode (upSwitchPin, INPUT);
  digitalWrite(upSwitchPin, HIGH); //подключаем подтягивающий резистор
  pinMode (downSwitchPin, INPUT);
  digitalWrite(downSwitchPin, HIGH); //подключаем подтягивающий резистор
  pinMode (cancelSwitchPin, INPUT);
  digitalWrite(cancelSwitchPin, HIGH); //подключаем подтягивающий резистор
  pinMode (okSwitchPin, INPUT);
  digitalWrite(okSwitchPin, HIGH); //подключаем подтягивающий резистор
  pinMode (buzzerPin, OUTPUT);
//setup reley pins as outputs
  pinMode(P1_PIN, OUTPUT);
  digitalWrite(P1_PIN, LOW); 
  pinMode(P2_PIN, OUTPUT);
  digitalWrite(P2_PIN, LOW); 
  pinMode(P3_PIN, OUTPUT);
  digitalWrite(P3_PIN, LOW); 
  pinMode(P4_PIN, OUTPUT);
  digitalWrite(P4_PIN, LOW); 
  
  myGLCD.InitLCD();
  myGLCD.clrScr();
  myGLCD.setFont(BigFont);
  myGLCD.setColor(VGA_GREEN);
  myGLCD.print("ARDUINO MEGA2560",CENTER, 100);
  myGLCD.setColor(VGA_RED);
  myGLCD.print("REWORK STATION",CENTER, 130);
  myGLCD.setColor(250, 180, 000);
  myGLCD.print("TFT-LCD 480*320 MAX6675",CENTER, 160);
  //Мелодия приветствия
  tone(buzzerPin, 523);
  delay(200);
  tone(buzzerPin, 659);
  delay(200);
  tone(buzzerPin, 784);
  delay(200);
  tone(buzzerPin, 1046);
  delay(200);
  noTone(buzzerPin);
  // wait for MAX chips to stabilize and splash screen
  delay(1000);
  myGLCD.clrScr();
  
//setup ssr pins as outputs
  pinMode(RelayPin1, OUTPUT);
  pinMode(RelayPin2, OUTPUT);  

  //windowStartTime = millis()
  nextRead1 = millis();

  myPID1.SetMode(AUTOMATIC);
  myPID2.SetMode(AUTOMATIC);
  myPID1.SetSampleTime(100); //задает частоту расчета выходного сигнала в милисекундах
  myPID2.SetSampleTime(100);
  }

int i = 0;
char buf[32];
void loop()
{
  //Считываем состояние кнопок управления
  upSwitchState = digitalRead(upSwitchPin);
  downSwitchState = digitalRead(downSwitchPin);
  cancelSwitchState = digitalRead(cancelSwitchPin);
  okSwitchState = digitalRead(okSwitchPin);
  //----------------------------------------------------------------------------------
  if (upSwitchState == HIGH) {
    upSwitchState = LOW;
  } else {
    upSwitchState = HIGH;
   
  }
  if (downSwitchState == HIGH) {
    downSwitchState = LOW;
  } else {
    downSwitchState = HIGH;
  }
  if (cancelSwitchState == HIGH) {
    cancelSwitchState = LOW;
  } else {
    cancelSwitchState = HIGH;
  }
  if (okSwitchState == HIGH) {
    okSwitchState = LOW;
  } else {
    okSwitchState = HIGH; 
  }
//-------------------------------------------------------------------------------

  //
  unsigned long currentMillis = millis();
      
  int SP1 = Setpoint1;
  int SP2 = Setpoint2;

//отключил бузер при нажатии на кнопки (нервирует), кому надо раскоментируйте ниже 6 строк
  //if (upSwitchState == HIGH || downSwitchState == HIGH || cancelSwitchState == HIGH || okSwitchState == HIGH)
  //{
     //tone(buzzerPin, 1045);
     //delay(100);
     //noTone(buzzerPin);
  //}
  if (reflowState == REFLOW_STATE_COMPLETE || alarmOn) {
    if (i < 2 && cancelSwitchState == LOW) {
      alarmOn = true;
      tone(buzzerPin, 1046);
      delay(100);
      noTone(buzzerPin);
      delay(100);
      tone(buzzerPin, 1046);
      delay(100);
      noTone(buzzerPin);
      delay(100);
      tone(buzzerPin, 1046);
      delay(100);
      noTone(buzzerPin);
      delay(100);
      tone(buzzerPin, 1046);
      delay(100);
      noTone(buzzerPin);
      delay(100);
      i++;
    }
    else {
      i = 0;
      alarmOn = false;
    }
  }

  switch (reflowState)
  {
    case REFLOW_STATE_IDLE:
     currentStep = 1;
     counter = 0;
     setpointRamp = 0;
     x = 1;             //устанавливаем переменную в исходное состояние
     flag = 0;         //после отановки профиля сбрасываем флаг
     Output1=0;
     Output2=0;

      if (millis() > nextRead1)
      {
        // Read thermocouples next sampling period
        nextRead1 += SENSOR_SAMPLING_TIME;
        Input1 = filter1(thermocouple1.readCelsius());
        Input2 = filter2(thermocouple2.readCelsius());
        tc1 = Input1;
        tc2 = Input2;

        sprintf (buf, "OK%03d%03d%03d%03d\r\n", int(Output1), int(Output2), tc1, tc2); // БУФЕР
        Serial.print(buf); // ВЫВОД
      
        if (isnan(Input1)) {
          myGLCD.setColor(VGA_BLACK);
          myGLCD.drawRoundRect(340,100,460,180);
          myGLCD.setFont(BigFont);
          myGLCD.setColor(VGA_RED);
          myGLCD.print("ERORR",360, 140);
        } else {
          myGLCD.setFont(SevenSegNumFont);
          myGLCD.setColor(VGA_SILVER);
          myGLCD.printNumI(tc1,345, 120,3,'0');
        }
        if (isnan(Input2)) {
          myGLCD.setFont(BigFont);
          myGLCD.setColor(VGA_RED);
          myGLCD.print("ERORR",360, 250);
        } else {
          myGLCD.setFont(SevenSegNumFont);
          myGLCD.setColor(VGA_SILVER);
          myGLCD.printNumI(tc2,345, 232,3,'0');
        }
      }

    //Настройка экрана Рабочий режим
      if (updateScreen) {
        myGLCD.clrScr();
        myGLCD.setFont(BigFontRus);
        myGLCD.setColor(VGA_SILVER);
        myGLCD.textRus("`",195, 123);
        myGLCD.textRus("`",195, 235);
        myGLCD.textRus("`",450, 123);
        myGLCD.textRus("`",450, 235);
        myGLCD.setColor(VGA_WHITE);
        myGLCD.textRus("ВЕРХНИЙ НАГРЕВАТЕЛЬ",90, 80);
        myGLCD.textRus("НИЖНИЙ НАГРЕВАТЕЛЬ",101, 194);
        myGLCD.setColor(VGA_SILVER);
        myGLCD.textRus("ПРОФИЛЬ",0, 5);
        myGLCD.drawRoundRect(2,108,478,179);
        myGLCD.drawRoundRect(2,221,478,291);
        myGLCD.drawRect(224,108,478,179);
        myGLCD.drawRect(224,221,478,291);
        myGLCD.textRus("УСТАН",5, 153);
        myGLCD.textRus("УСТАН",5, 266);
        myGLCD.textRus("НАГРЕВ",235, 153);
        myGLCD.textRus("НАГРЕВ",235, 266);
        updateScreen = false;
      }
        myGLCD.setFont(BigFont);
        myGLCD.setColor(250, 180, 000);
        myGLCD.printNumI(currentProfile,120, 5);
        myGLCD.setFont(SevenSegNumFont);
        myGLCD.setColor(VGA_SILVER);
        myGLCD.printNumI(temperatureStep[editStep],90, 120,3,'0');
        myGLCD.printNumI(SP2,90, 232,3,'0');
        windowStartTime = millis();

       if (currentProfile == 1) profileName = 1;
       if (currentProfile == 2) profileName = 2;
       if (currentProfile == 3) profileName = 3;
       if (currentProfile == 4) profileName = 4;
       if (currentProfile == 5) profileName = 5;
       
        if (profileName == 1){
        myGLCD.setFont(BigFontRus);
        myGLCD.setColor(VGA_LIME);
        myGLCD.textRus("СНЯТИЕ ЧИПА БЕССВИНЕЦ",75, 40);
        }
        if (profileName == 2){
        myGLCD.setFont(BigFontRus);
        myGLCD.setColor(VGA_LIME);
        myGLCD.textRus(" СНЯТИЕ ЧИПА СВИНЕЦ   ",75, 40);
        }
        if (profileName == 3){
        myGLCD.setFont(BigFontRus);
        myGLCD.setColor(VGA_LIME);
        myGLCD.textRus("УС-ВКА ЧИПА БЕССВИНЕЦ",75, 40);
        }
        if (profileName == 4){
        myGLCD.setFont(BigFontRus);
        myGLCD.setColor(VGA_LIME);
        myGLCD.textRus(" УС-ВКА ЧИПА СВИНЕЦ   ",75, 40);
        }
        if (profileName == 5){
        myGLCD.setFont(BigFontRus);
        myGLCD.setColor(VGA_LIME);
        myGLCD.textRus("   НИЖНИЙ ПОДОГРЕВ     ",75, 40);
      }
      
      if (upSwitchState == HIGH && ( millis() - ms_button)>500)//if up switch is pressed go to next profile
      {
        ms_button =  millis();
        currentProfile = currentProfile + 1;
        if (currentProfile >= 6)//if currentProfile = 5 and up is pressed go back to profile 1
        {
          currentProfile = 1;
        }
      }
      if (downSwitchState == HIGH && ( millis() - ms_button)>500)//same as above just go down one profile
      {
        ms_button =  millis();
        currentProfile = currentProfile - 1;
        if (currentProfile <= 0)
        {
          currentProfile = 5;
        }
      }
      loadProfile();//call the loadProfile function to load from eeprom

//фиксируем момент нажатия кнопки "ОК" + защита от дребезга
      if (okSwitchState == HIGH && !button_state && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        button_state = true;
        button_long_state = false;
      }
//держим "ОК" 3секунды и заходим в меню настроек      
      if (okSwitchState == HIGH && !button_long_state && ( millis() - ms_button)>3000)
      {
        button_long_state = true;
        button_state = false;
        reflowState = REFLOW_STATE_MENU_STEPS;
        //update next screen
        updateScreen = true;
      }
//включаем пайку, кнопка сработает после отпускания "ОК"       
      if (okSwitchState == LOW && button_state && ( millis() - ms_button)>100)
      {        
        ms_button =  millis();
        button_state = false;
        tone(buzzerPin, 1045, 500);  //звуковой сигнал при старте профиля
        //update next screen
        updateScreen = true;
        reflowStatus = REFLOW_STATUS_ON;
        reflowState = REFLOW_STATE_STEP_RAMP;
      }
      break;
//устанавливаем количество шагов профиля
    case REFLOW_STATE_MENU_STEPS:
      if (updateScreen){
        myGLCD.clrScr();
        myGLCD.setFont(BigFontRus);
        myGLCD.setColor(VGA_RED);
        myGLCD.textRus("ПР-ЛЬ",0, 20);
        myGLCD.textRus("НИЖНИЙ НАГРЕВАТЕЛЬ",90, 60);
        myGLCD.setColor(VGA_LIME);
        myGLCD.textRus("К-ВО ШАГОВ",10, 110);
        myGLCD.setColor(VGA_SILVER);
        myGLCD.textRus("РАЗМЕР НИЗА",260, 110);
        myGLCD.textRus("ТЕМПЕРАТУРА НИЗ",5, 180);
        myGLCD.textRus("МОЩНОСТЬ НИЗ",280, 180);
        myGLCD.textRus("МИН",320, 220);
        myGLCD.textRus("МАХ",320, 260);
        myGLCD.textRus("`",180, 223);
        myGLCD.textRus("%",430, 220);
        myGLCD.textRus("%",430, 260);
        myGLCD.printNumI(tableSize,450, 110);
        myGLCD.printNumI(min_pwr_BOTTOM,380, 220,3,'0');
        myGLCD.printNumI(max_pwr_BOTTOM,380, 260,3,'0');
        myGLCD.setFont(SevenSegNumFont);
        myGLCD.printNumI(SP2,75, 220);
        myGLCD.setColor(250, 180, 000);
        myGLCD.drawRoundRect(5,103,200,131);
        myGLCD.setFont(BigFontRus);
        myGLCD.printNumI(currentProfile,88, 20);
        myGLCD.printNumI(profileSteps,180, 110);
        updateScreen = false;
      }
        myGLCD.setColor(250, 180, 000);
        myGLCD.printNumI(profileSteps,180, 110);

        if (profileName == 1){
        myGLCD.setFont(BigFontRus);
        myGLCD.setColor(VGA_LIME);
        myGLCD.textRus("СНЯТИЕ ЧИПА БЕССВИНЕЦ",120, 20);
        }
        if (profileName == 2){
        myGLCD.setFont(BigFontRus);
        myGLCD.setColor(VGA_LIME);
        myGLCD.textRus("СНЯТИЕ ЧИПА СВИНЕЦ",120, 20);
        }
        if (profileName == 3){
        myGLCD.setFont(BigFontRus);
        myGLCD.setColor(VGA_LIME);
        myGLCD.textRus("УС-ВКА ЧИПА БЕССВИНЕЦ",120, 20);
        }
        if (profileName == 4){
        myGLCD.setFont(BigFontRus);
        myGLCD.setColor(VGA_LIME);
        myGLCD.textRus("УС-ВКА ЧИПА СВИНЕЦ",120, 20);
        }
        if (profileName == 5){
        myGLCD.setFont(BigFontRus);
        myGLCD.setColor(VGA_LIME);
        myGLCD.textRus("НИЖНИЙ ПОДОГРЕВ",120, 20);
      }

      if (upSwitchState == HIGH && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        profileSteps = profileSteps + 1;
        if (profileSteps >= 4)
        {
          profileSteps = 1;
        }
      }
      if (downSwitchState == HIGH && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        profileSteps = profileSteps - 1;
        if (profileSteps <= 0) 
        {
          profileSteps = 3;
        }
      }
      
      if (okSwitchState == LOW && button_long_state) button_long_state = false; //чтобы после входа в меню не кнопка "ОК" не считалась нажатой
      
      if (okSwitchState == HIGH && !button_state && !button_long_state && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        button_state = true;
      }
      if (okSwitchState == LOW && button_state && ( millis() - ms_button)>100)
      {        
        ms_button =  millis();
        button_state = false;
        EEPROM.write((currentProfile - 1) * 29, profileSteps);
        updateScreen = true;
        reflowState = REFLOW_STATE_MENU_TABLE_SIZE;
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button)>60)
      {
        ms_button =  millis();
        updateScreen = true;
        reflowState = REFLOW_STATE_IDLE;
      }
      break;
//устанавливаем размер стола
    case REFLOW_STATE_MENU_TABLE_SIZE:
      if (updateScreen){
        myGLCD.setColor(VGA_BLACK);
        myGLCD.drawRoundRect(5,103,200,131);
        myGLCD.setFont(BigFontRus);
        myGLCD.setColor(VGA_LIME);
        myGLCD.textRus("РАЗМЕР НИЗА",260, 110);
        myGLCD.setColor(VGA_SILVER);
        myGLCD.textRus("К-ВО ШАГОВ",10, 110);
        myGLCD.printNumI(profileSteps,180, 110);
        myGLCD.setFont(SevenSegNumFont);
        myGLCD.setColor(250, 180, 000);
        myGLCD.drawRoundRect(250,103,475,131);
        myGLCD.setFont(BigFontRus);
        myGLCD.printNumI(tableSize,450, 110);
        updateScreen = false;
      }
        myGLCD.printNumI(tableSize,450, 110);

      if (upSwitchState == HIGH && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        tableSize = tableSize + 1;
        if (tableSize >= 4)
        {
          tableSize = 1;
        }
      }
      if (downSwitchState == HIGH && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        tableSize = tableSize - 1;
        if (tableSize <= 0) 
        {
          tableSize = 3;
        }
      }      
      if (okSwitchState == HIGH && !button_state && !button_long_state && ( millis() - ms_button)>120)
      {
        ms_button =  millis();
        button_state = true;
      }
      if (okSwitchState == LOW && button_state && ( millis() - ms_button)>100)
      {        
        ms_button =  millis();
        button_state = false;
        EEPROM.write(((currentProfile - 1) * 6 + 202), tableSize);
        updateScreen = true;
        reflowState = REFLOW_STATE_MENU_BOTTOM_HEAT;
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button)>60)
      {
        ms_button =  millis();
        updateScreen = true;
        reflowState = REFLOW_STATE_IDLE;
      }
      break;
//устанавливаем температуру "Нижнего Нагревателя"
    case REFLOW_STATE_MENU_BOTTOM_HEAT:
      if (updateScreen) {
        myGLCD.setFont(BigFontRus);
        myGLCD.setColor(VGA_BLACK);
        myGLCD.drawRoundRect(250,103,475,131);
        myGLCD.setColor(VGA_SILVER);
        myGLCD.textRus("РАЗМЕР НИЗА",260, 110);
        myGLCD.printNumI(tableSize,450, 110);
        myGLCD.setColor(VGA_LIME);
        myGLCD.textRus("ТЕМПЕРАТУРА НИЗ",5, 180);
        myGLCD.setColor(250, 180, 000);
        myGLCD.drawRoundRect(2,175,258,203);
        updateScreen = false;
      }
        myGLCD.setFont(SevenSegNumFont);
        myGLCD.printNumI(SP2,75, 220,3,'0');

      if (upSwitchState == HIGH && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        Setpoint2 = Setpoint2 + 1;
        if (Setpoint2 >= 250)
        {
          Setpoint2 = 250;
        }
      }
      if (downSwitchState == HIGH && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        Setpoint2 = Setpoint2 - 1;
        if (Setpoint2 <= 100)
        {
          Setpoint2 = 100;
        }
      }
      if (okSwitchState == HIGH && !button_state && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        button_state = true;
      }
      if (okSwitchState == LOW && button_state && ( millis() - ms_button)>100)
      {        
        ms_button =  millis();
        button_state = false;
        EEPROM.write((currentProfile - 1) * 29 + 1, Setpoint2);
        updateScreen = true;
        reflowState = REFLOW_STATE_MENU_BOTTOM_PWR_MIN;
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button)>60)
      {
        ms_button =  millis();
        updateScreen = true;
        reflowState = REFLOW_STATE_IDLE;
      }
      break;
 //устанавливаем минимальную мощность "НИЖНЕГО НАГРЕВАТЕЛЯ"     
     case REFLOW_STATE_MENU_BOTTOM_PWR_MIN:    
      if (updateScreen){
        myGLCD.setFont(BigFontRus);        
        myGLCD.setColor(VGA_BLACK);
        myGLCD.drawRoundRect(2,175,258,203);
        myGLCD.setColor(VGA_LIME);
        myGLCD.textRus("МОЩНОСТЬ НИЗ",280, 180);
        myGLCD.textRus("МИН",320, 220);
        myGLCD.setColor(VGA_SILVER);
        myGLCD.textRus("ТЕМПЕРАТУРА НИЗ",5, 180);
        myGLCD.setFont(SevenSegNumFont);
        myGLCD.printNumI(SP2,75, 220);
        myGLCD.setColor(250, 180, 000);
        myGLCD.drawRoundRect(275,175,475,203);
        updateScreen = false;
      }
        myGLCD.setFont(BigFontRus);
        myGLCD.printNumI(min_pwr_BOTTOM,380, 220,3,'0');
      
       if (upSwitchState == HIGH && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        min_pwr_BOTTOM = min_pwr_BOTTOM + 1;
        if (min_pwr_BOTTOM >= 100)
        {
          min_pwr_BOTTOM = 100;
        }
      }
      if (downSwitchState == HIGH && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        min_pwr_BOTTOM = min_pwr_BOTTOM - 1;
        if (min_pwr_BOTTOM <= 0)
        {
         min_pwr_BOTTOM = 0;
        }
      }
      if (okSwitchState == HIGH && !button_state && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        button_state = true;
      }
      if (okSwitchState == LOW && button_state && ( millis() - ms_button)>100)
      {        
        ms_button =  millis();
        button_state = false;
        EEPROM.write(((currentProfile - 1) * 6 + 203), min_pwr_BOTTOM);
        updateScreen = true;
        reflowState = REFLOW_STATE_MENU_BOTTOM_PWR_MAX;
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button)>50)
      {
        ms_button =  millis();
        updateScreen = true;
        reflowState = REFLOW_STATE_IDLE;
      }
      break;

 //устанавливаем максимальную мощность "НИЖНЕГО НАГРЕВАТЕЛЯ"     
     case REFLOW_STATE_MENU_BOTTOM_PWR_MAX:    
      if (updateScreen){
        myGLCD.setFont(BigFontRus);        
        myGLCD.setColor(VGA_BLACK);
        myGLCD.drawRoundRect(2,175,258,203);
        myGLCD.setColor(VGA_LIME);
        myGLCD.textRus("МОЩНОСТЬ НИЗ",280, 180);
        myGLCD.textRus("МАХ",320, 260);
        myGLCD.setColor(VGA_SILVER);
        myGLCD.textRus("МИН",320, 220);
        myGLCD.printNumI(min_pwr_BOTTOM,380, 220,3,'0');
        myGLCD.setColor(250, 180, 000);
        myGLCD.drawRoundRect(275,175,475,203);
        updateScreen = false;
      }
        myGLCD.printNumI(max_pwr_BOTTOM,380, 260,3,'0');
      
       if (upSwitchState == HIGH && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        max_pwr_BOTTOM = max_pwr_BOTTOM + 1;
        if (max_pwr_BOTTOM >= 100)
        {
          max_pwr_BOTTOM = 100;
        }
      }
      if (downSwitchState == HIGH && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        max_pwr_BOTTOM = max_pwr_BOTTOM - 1;
        if (max_pwr_BOTTOM <= 0)
        {
         max_pwr_BOTTOM = 0;
        }
      }
      if (okSwitchState == HIGH && !button_state && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        button_state = true;
      }
      if (okSwitchState == LOW && button_state && ( millis() - ms_button)>100)
      {        
        ms_button =  millis();
        button_state = false;
        EEPROM.write(((currentProfile - 1) * 6 + 204), max_pwr_BOTTOM);
        updateScreen = true;
        reflowState = REFLOW_STATE_MENU_STEP_RAMP;
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button)>50)
      {
        ms_button =  millis();
        updateScreen = true;
        reflowState = REFLOW_STATE_IDLE;
      }
      break;
      
//устанавливаем скорость нагрева "Верхним Нагревателем"
    case REFLOW_STATE_MENU_STEP_RAMP:
      if (updateScreen) {
        myGLCD.clrScr();
        myGLCD.setFont(BigFontRus);
        myGLCD.setColor(VGA_RED);
        myGLCD.textRus("ПР-ЛЬ",0, 20);
        myGLCD.textRus("ВЕРХНИЙ НАГРЕВАТЕЛЬ",120, 55);
        myGLCD.setColor(VGA_LIME);
        myGLCD.textRus("СКОРОСТЬ НАГРЕВА ВЕРХОМ",15, 90);
        myGLCD.setColor(VGA_SILVER);
        myGLCD.textRus(" ШАГ:",0, 55);
        myGLCD.textRus("ТЕМП-РА ВЕРХ",5, 140);
        myGLCD.textRus("МОЩНОСТЬ ВЕРХ",260, 140);
        myGLCD.textRus("МИН",310, 180);
        myGLCD.textRus("МАХ",310, 220);
        myGLCD.printNumI(min_pwr_TOPStep[editStep],370, 180,3,'0');
        myGLCD.printNumI(max_pwr_TOPStep[editStep],370, 220,3,'0');
        myGLCD.textRus("ВРЕМЯ ПЕРЕХОДА НА СЛЕД ШАГ",5, 270);
        myGLCD.textRus("с",460, 90);
        myGLCD.textRus("с",462, 270);
        myGLCD.textRus("`",160, 183);
        myGLCD.textRus("%",420, 180);
        myGLCD.textRus("%",420, 220);
        myGLCD.setColor(250, 180, 000);
        myGLCD.drawRoundRect(12,85,390,112);
        myGLCD.printNumI(currentProfile,88, 20);
        myGLCD.setColor(VGA_SILVER);
        myGLCD.setFont(SevenSegNumFont);
        myGLCD.printNumI(temperatureStep[editStep],55, 180,3,'0');
        myGLCD.setFont(BigFont);
        myGLCD.printNumI(dwellTimerStep[editStep],427, 270,2,'0');
        myGLCD.setColor(250, 180, 000);
        myGLCD.printNumI(editStep + 1,70, 55);
        updateScreen = false;
      }
        myGLCD.setColor(250, 180, 000);
        myGLCD.printNumF(rampRateStep[editStep] * 0.1,0, 405, 90);
        
        if (profileName == 1){
        myGLCD.setFont(BigFontRus);
        myGLCD.setColor(VGA_LIME);
        myGLCD.textRus("СНЯТИЕ ЧИПА БЕССВИНЕЦ",120, 20);
        }
        if (profileName == 2){
        myGLCD.setFont(BigFontRus);
        myGLCD.setColor(VGA_LIME);
        myGLCD.textRus("СНЯТИЕ ЧИПА СВИНЕЦ",120, 20);
        }
        if (profileName == 3){
        myGLCD.setFont(BigFontRus);
        myGLCD.setColor(VGA_LIME);
        myGLCD.textRus("УС-ВКА ЧИПА БЕССВИНЕЦ",120, 20);
        }
        if (profileName == 4){
        myGLCD.setFont(BigFontRus);
        myGLCD.setColor(VGA_LIME);
        myGLCD.textRus("УС-ВКА ЧИПА СВИНЕЦ",120, 20);
        }
        if (profileName == 5){
        myGLCD.setFont(BigFontRus);
        myGLCD.setColor(VGA_LIME);
        myGLCD.textRus("НИЖНИЙ ПОДОГРЕВ",120, 20);
        myGLCD.setColor(250, 180, 000);
      }
             
      if (upSwitchState == HIGH && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        rampRateStep[editStep] = rampRateStep[editStep] + 1.;
        if (rampRateStep[editStep] >= 30.)
        {
          rampRateStep[editStep] = 30.;  //максимальная скорость роста температуры от 0.1с. до 3с.
        }
      }
      if (downSwitchState == HIGH && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        rampRateStep[editStep] = rampRateStep[editStep] - 1.;
        if (rampRateStep[editStep] <= 1.)
        {
          rampRateStep[editStep] = 1.;  //минимальная скорость роста температуры от 3с. до 0.1с.
        }
      }
      if (okSwitchState == HIGH && !button_state && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        button_state = true;
      }
      if (okSwitchState == LOW && button_state && ( millis() - ms_button)>100)
      {        
        ms_button =  millis();
        button_state = false;
        updateScreen = true;
        if (editStep + 1 == profileSteps) {
          editStep = 0;
          reflowState = REFLOW_STATE_MENU_STEP_TARGET;
        }
        else {
          editStep++;
        }
        for (int i = 0; i < 3; i + 1) {
          EEPROM.write(((currentProfile - 1) * 29 + i + 2), rampRateStep[i]);
          i++;
        }
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button)>60)
      {
        ms_button =  millis();
        updateScreen = true;
        editStep = 0;
        reflowState = REFLOW_STATE_IDLE;
      }
      break;
 //устанавливаем температуру "Верхнего Нагревателя"
    case REFLOW_STATE_MENU_STEP_TARGET:
      if (updateScreen) {
        myGLCD.setFont(BigFontRus);
        myGLCD.setColor(VGA_BLACK);
        myGLCD.drawRoundRect(12,85,390,112);
        myGLCD.setColor(VGA_SILVER);
        myGLCD.textRus("СКОРОСТЬ НАГРЕВА ВЕРХОМ",15, 90);
        myGLCD.textRus("с",455, 90);
        myGLCD.printNumF(rampRateStep[editStep] * 0.1,0, 405, 90);
        myGLCD.setColor(VGA_LIME);
        myGLCD.textRus("ТЕМП-РА ВЕРХ",5, 140); 
        myGLCD.setColor(250, 180, 000);
        myGLCD.drawRoundRect(2,131,200,160);
        myGLCD.printNumI(editStep + 1,70, 55);
        myGLCD.setFont(SevenSegNumFont);
        myGLCD.printNumI(temperatureStep[editStep],55, 180,3,'0');       
        updateScreen = false;
      }
        myGLCD.printNumI(temperatureStep[editStep],55, 180,3,'0');
        
      if (upSwitchState == HIGH && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        temperatureStep[editStep] = temperatureStep[editStep] + 1;
        if (temperatureStep[editStep] >= 250)
        {
          temperatureStep[editStep] = 250;
        }
      }
      if (downSwitchState == HIGH && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        temperatureStep[editStep] = temperatureStep[editStep] - 1;
        if (temperatureStep[editStep] <= 0)
        {
          temperatureStep[editStep] = 0;
        }
        if (temperatureStep[editStep] <= 99)
        {
         myGLCD.printNumI(temperatureStep[editStep],180, 160);
        }
        if (temperatureStep[editStep] <= 9)
        {
         myGLCD.printNumI(temperatureStep[editStep],180, 160);
        }
      }
      if (okSwitchState == HIGH && !button_state && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        button_state = true;
      }
      if (okSwitchState == LOW && button_state && ( millis() - ms_button)>100)
      {        
        ms_button =  millis();
        button_state = false;
        updateScreen = true;
        if (editStep + 1 == profileSteps) {
          editStep = 0;
          reflowState = REFLOW_STATE_MENU_TOP_PWR_MIN;
        }
        else {
          editStep++;
        }
        for (int i = 0; i < 3; i + 1) {
          EEPROM.write(((currentProfile - 1) * 29 + i + 8), temperatureStep[i]);
          i++;
        }
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button)>60)
      {
        ms_button =  millis();
        updateScreen = true;
        editStep = 0;
        reflowState = REFLOW_STATE_IDLE;
      }
      break;
 //устанавливаем минимальную мощность "ВЕРХНЕГО НАГРЕВАТЕЛЯ"      
     case REFLOW_STATE_MENU_TOP_PWR_MIN:      
      if (updateScreen)
      {
        myGLCD.setFont(BigFontRus);        
        myGLCD.setColor(VGA_BLACK);
        myGLCD.drawRoundRect(2,131,200,160);
        myGLCD.setColor(VGA_SILVER);
        myGLCD.textRus("ТЕМП-РА ВЕРХ",5, 140);
        myGLCD.printNumF(rampRateStep[editStep] * 0.1,0, 405, 90);
        myGLCD.setFont(SevenSegNumFont);
        myGLCD.printNumI(temperatureStep[editStep],55, 180,3,'0');
        myGLCD.setFont(BigFontRus);
        myGLCD.setColor(VGA_LIME);
        myGLCD.textRus("МОЩНОСТЬ ВЕРХ",260, 140);
        myGLCD.textRus("МИН",310, 180);
        myGLCD.setColor(250, 180, 000);
        myGLCD.printNumI(editStep + 1,70, 55);
        myGLCD.drawRoundRect(255,131,470,160);
        myGLCD.setFont(BigFontRus);
        myGLCD.printNumI(min_pwr_TOPStep[editStep],370, 180,3,'0');
        updateScreen = false;
      }
        myGLCD.printNumI(min_pwr_TOPStep[editStep],370, 180,3,'0');
      
       if (upSwitchState == HIGH && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        min_pwr_TOPStep[editStep] = min_pwr_TOPStep[editStep] + 1;
        if (min_pwr_TOPStep[editStep] >= 100)
        {
          min_pwr_TOPStep[editStep] = 100;
        }
      }
      if (downSwitchState == HIGH && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        min_pwr_TOPStep[editStep] = min_pwr_TOPStep[editStep] - 1;
        if (min_pwr_TOPStep[editStep] <= 0)
        {
         min_pwr_TOPStep[editStep] = 0;
        }
      }
      if (okSwitchState == HIGH && !button_state && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        button_state = true;
      }
      if (okSwitchState == LOW && button_state && ( millis() - ms_button)>100)
      {        
        ms_button =  millis();
        button_state = false;
        updateScreen = true;
        if (editStep + 1 == profileSteps) {
        editStep = 0;
        reflowState = REFLOW_STATE_MENU_TOP_PWR_MAX;
        }
        else {
          editStep++;
        }
        for (int i = 0; i < 3; i + 1) {
          EEPROM.write(((currentProfile - 1) * 29 + i + 11), min_pwr_TOPStep[i]);
          i++;
        }
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button)>60)
      {
        ms_button =  millis();
        updateScreen = true;
        editStep = 0;
        reflowState = REFLOW_STATE_IDLE;
      }
      break;

 //устанавливаем максимальную мощность "ВЕРХНЕГО НАГРЕВАТЕЛЯ"      
     case REFLOW_STATE_MENU_TOP_PWR_MAX:      
      if (updateScreen)
      {
        myGLCD.setFont(BigFontRus);        
        myGLCD.setColor(VGA_BLACK);
        myGLCD.drawRoundRect(2,131,200,160);
        myGLCD.setColor(VGA_SILVER);
        myGLCD.textRus("МИН",310, 180);
        myGLCD.printNumI(min_pwr_TOPStep[editStep],370, 180,3,'0');
        myGLCD.printNumF(rampRateStep[editStep] * 0.1,0, 405, 90);
        myGLCD.setFont(SevenSegNumFont);
        myGLCD.printNumI(temperatureStep[editStep],55, 180,3,'0');
        myGLCD.setFont(BigFontRus);
        myGLCD.setColor(VGA_LIME);
        myGLCD.textRus("МАХ",310, 220);
        myGLCD.setColor(250, 180, 000);
        myGLCD.printNumI(editStep + 1,70, 55);
        myGLCD.drawRoundRect(255,131,470,160);
        myGLCD.setFont(BigFontRus);
        myGLCD.printNumI(max_pwr_TOPStep[editStep],370, 220,3,'0');
        updateScreen = false;
      }
        myGLCD.printNumI(max_pwr_TOPStep[editStep],370, 220,3,'0');
      
       if (upSwitchState == HIGH && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        max_pwr_TOPStep[editStep] = max_pwr_TOPStep[editStep] + 1;
        if (max_pwr_TOPStep[editStep] >= 100)
        {
          max_pwr_TOPStep[editStep] = 100;
        }
      }
      if (downSwitchState == HIGH && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        max_pwr_TOPStep[editStep] = max_pwr_TOPStep[editStep] - 1;
        if (max_pwr_TOPStep[editStep] <= 0)
        {
         max_pwr_TOPStep[editStep] = 0;
        }
      }
      if (okSwitchState == HIGH && !button_state && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        button_state = true;
      }
      if (okSwitchState == LOW && button_state && ( millis() - ms_button)>100)
      {        
        ms_button =  millis();
        button_state = false;
        updateScreen = true;
        if (editStep + 1 == profileSteps) {
        editStep = 0;
        reflowState = REFLOW_STATE_MENU_STEP_DWELL;
        }
        else {
          editStep++;
        }
        for (int i = 0; i < 3; i + 1) {
          EEPROM.write(((currentProfile - 1) * 29 + i + 14), max_pwr_TOPStep[i]);
          i++;
        }
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button)>60)
      {
        ms_button =  millis();
        updateScreen = true;
        editStep = 0;
        reflowState = REFLOW_STATE_IDLE;
      }
      break;
      
//устанавливаем время перехода на следующий шаг
    case REFLOW_STATE_MENU_STEP_DWELL:
      if (updateScreen) {
        myGLCD.setFont(BigFontRus);
        myGLCD.setColor(VGA_BLACK);
        myGLCD.drawRoundRect(255,131,470,160);
        myGLCD.setColor(VGA_SILVER);
        myGLCD.textRus("МОЩНОСТЬ ВЕРХ",260, 140);
        myGLCD.textRus("МАХ",310, 220);
        myGLCD.printNumI(max_pwr_TOPStep[editStep],370, 220,3,'0');
        myGLCD.printNumF(rampRateStep[editStep] * 0.1,0, 405, 90);
        myGLCD.setFont(SevenSegNumFont);
        myGLCD.printNumI(temperatureStep[editStep],55, 180,3,'0');
        myGLCD.setFont(BigFontRus);
        myGLCD.setColor(VGA_LIME);
        myGLCD.textRus("ВРЕМЯ ПЕРЕХОДА НА СЛЕД ШАГ",5, 270);
        myGLCD.setColor(250, 180, 000);
        myGLCD.drawRoundRect(2,263,420,290);
        myGLCD.printNumI(editStep + 1,70, 55);
        myGLCD.printNumI(dwellTimerStep[editStep],427, 270,2,'0');
        updateScreen = false;
       }
        myGLCD.printNumI(dwellTimerStep[editStep],427, 270,2,'0');

      if (upSwitchState == HIGH && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        dwellTimerStep[editStep] = dwellTimerStep[editStep] + 5;
        if (dwellTimerStep[editStep] >= 90)
        {
          dwellTimerStep[editStep] = 90;
        }
      }
      if (downSwitchState == HIGH && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        dwellTimerStep[editStep] = dwellTimerStep[editStep] - 5;
        if (dwellTimerStep[editStep] <= 0)
        {
          dwellTimerStep[editStep] = 0;
        }
      }
      if (okSwitchState == HIGH && !button_state && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        button_state = true;
      }
      if (okSwitchState == LOW && button_state && ( millis() - ms_button)>100)
      {        
        ms_button =  millis();
        button_state = false;
        for (int i = 0; i < 3; i + 1) {
          EEPROM.write(((currentProfile - 1) * 29 + i + 5), dwellTimerStep[i] / 5);
          i++;
        }
        updateScreen = true;
        if (editStep + 1 == profileSteps) {
          editStep = 0;
          reflowState = REFLOW_STATE_MENU_BOTTOM_P;
        }
        else {
          editStep++;
        }
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button)>60)
      {
        ms_button =  millis();
        updateScreen = true;
        editStep = 0;
        reflowState = REFLOW_STATE_IDLE;
      }
      break;
//настройка "ПИД" нижнего нагревателя
    case REFLOW_STATE_MENU_BOTTOM_P:
      if (updateScreen) {
        myGLCD.clrScr();
        myGLCD.setFont(BigFontRus);
        myGLCD.setColor(VGA_RED);
        myGLCD.textRus("НАСТРОЙКА ПИД",130, 5);
        myGLCD.setColor(VGA_LIME);
        myGLCD.textRus("НИЖНИЙ НАГРЕВАТЕЛЬ",90, 35);
        myGLCD.setColor(VGA_SILVER);
        myGLCD.textRus("ВЕРХНИЙ НАГРЕВАТЕЛЬ",85, 170);
        myGLCD.setFont(SevenSegNumFont);
        myGLCD.printNumI(ki2,200, 100,3,'0');
        myGLCD.printNumI(kd2,375, 100,3,'0');
        myGLCD.setFont(BigFont);
        myGLCD.print("I=",230, 70);
        myGLCD.print("D=",405, 70);
        myGLCD.print("P=",55, 205);
        myGLCD.print("I=",230, 205);
        myGLCD.print("D=",405, 205);
        myGLCD.setColor(250, 180, 000);
        myGLCD.print("P=",55, 70);
        myGLCD.setFont(SevenSegNumFont);
        myGLCD.drawRoundRect(80,28,390,55);
        myGLCD.printNumI(kp2,25, 100,3,'0');
        updateScreen = false;
      }
        myGLCD.printNumI(kp2,25, 100,3,'0');
        
      if (upSwitchState == HIGH && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        kp2 = kp2 + 1;
        if (kp2 >= 500)
        {
          kp2 = 500;
        }
      }
      if (downSwitchState == HIGH && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        kp2 = kp2 - 1;
        if (kp2 <= 0)
        {
          kp2 = 0;
        }
      }
      if (okSwitchState == HIGH && !button_state && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        button_state = true;
      }
      if (okSwitchState == LOW && button_state && ( millis() - ms_button)>100)
      {        
        ms_button =  millis();
        button_state = false;
        EEPROM.write(((currentProfile - 1) * 6 + 175), kp2);
        updateScreen = true;
        reflowState = REFLOW_STATE_MENU_BOTTOM_I;
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button)>60)
      {
        ms_button =  millis();
        updateScreen = true;
        reflowState = REFLOW_STATE_IDLE;
      }
      break;
    case REFLOW_STATE_MENU_BOTTOM_I:
        myGLCD.setFont(BigFont);
        myGLCD.setColor(250, 180, 000);
        myGLCD.print("I=",230, 70);
        myGLCD.setColor(VGA_SILVER);
        myGLCD.print("P=",55, 70);
        myGLCD.setFont(SevenSegNumFont);
        myGLCD.printNumI(kp2,25, 100,3,'0');
        myGLCD.setColor(250, 180, 000);
        myGLCD.printNumI(ki2,200, 100,3,'0');       
      if (upSwitchState == HIGH && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        ki2 = ki2 + 1;
        if (ki2 >= 500)
        {
          ki2 = 500;
        }
      }
      if (downSwitchState == HIGH && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        ki2 = ki2 - 1;
        if (ki2 <= 0)
        {
          ki2 = 0;
        }
      }
      if (okSwitchState == HIGH && !button_state && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        button_state = true;
      }
      if (okSwitchState == LOW && button_state && ( millis() - ms_button)>100)
      {        
        ms_button =  millis();
        button_state = false;
        EEPROM.write(((currentProfile - 1) * 6 + 176), ki2);
        reflowState = REFLOW_STATE_MENU_BOTTOM_D;
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button)>60)
      {
        ms_button =  millis();
        reflowState = REFLOW_STATE_IDLE;
      }
      break;
    case REFLOW_STATE_MENU_BOTTOM_D:
        myGLCD.setFont(BigFont);
        myGLCD.setColor(250, 180, 000);
        myGLCD.print("D=",405, 70);
        myGLCD.setColor(VGA_SILVER);
        myGLCD.print("I=",230, 70);
        myGLCD.setFont(SevenSegNumFont);
        myGLCD.printNumI(ki2,200, 100,3,'0');
        myGLCD.setColor(250, 180, 000);
        myGLCD.printNumI(kd2,375, 100,3,'0');
        
      if (upSwitchState == HIGH && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        kd2 = kd2 + 1;
        if (kd2 >= 500)
        {
          kd2 = 500;
        }
      }
      if (downSwitchState == HIGH && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        kd2 = kd2 - 1;
        if (kd2 <= 0)
        {
          kd2 = 0;
        }
      }
      if (okSwitchState == HIGH && !button_state && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        button_state = true;
      }
      if (okSwitchState == LOW && button_state && ( millis() - ms_button)>100)
      {        
        ms_button =  millis();
        button_state = false;
        EEPROM.write(((currentProfile - 1) * 6 + 177), kd2);
        reflowState = REFLOW_STATE_MENU_TOP_P;
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button)>60)
      {
        ms_button =  millis();
        reflowState = REFLOW_STATE_IDLE;
      }
      break;
//настройка "ПИД" верхнего нагревателя
    case REFLOW_STATE_MENU_TOP_P:
      if (updateScreen) {
        myGLCD.setFont(BigFontRus);
        myGLCD.setColor(250, 180, 000);
        myGLCD.print("P=",55, 205);
        myGLCD.setColor(VGA_LIME);
        myGLCD.textRus("ВЕРХНИЙ НАГРЕВАТЕЛЬ",85, 170);
        myGLCD.setColor(VGA_SILVER);
        myGLCD.textRus("НИЖНИЙ НАГРЕВАТЕЛЬ",90, 35);
        myGLCD.setFont(BigFont);
        myGLCD.print("D=",405, 70);
        myGLCD.setFont(SevenSegNumFont);
        myGLCD.printNumI(kd2,375, 100,3,'0');
        myGLCD.printNumI(ki1,200, 240,3,'0');
        myGLCD.printNumI(kd1,375, 240,3,'0');
        myGLCD.setColor(VGA_BLACK);
        myGLCD.drawRoundRect(80,28,390,55);
        myGLCD.setColor(250, 180, 000);
        myGLCD.drawRoundRect(80,163,394,190);
        myGLCD.printNumI(kp1,25, 240,3,'0');
        updateScreen = false;
      }
        myGLCD.printNumI(kp1,25, 240,3,'0');
        
      if (upSwitchState == HIGH && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        kp1 = kp1 + 1;
        if (kp1 >= 500)
        {
          kp1 = 500;
        }
      }
      if (downSwitchState == HIGH && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        kp1 = kp1 - 1;
        if (kp1 <= 0)
        {
          kp1 = 0;
        }
      }
      if (okSwitchState == HIGH && !button_state && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        button_state = true;
      }
      if (okSwitchState == LOW && button_state && ( millis() - ms_button)>100)
      {        
        ms_button =  millis();
        button_state = false;
        EEPROM.write(((currentProfile - 1) * 6 + 172), kp1);
        updateScreen = true;
        reflowState = REFLOW_STATE_MENU_TOP_I;
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button)>60)
      {
        ms_button =  millis();
        updateScreen = true;
        reflowState = REFLOW_STATE_IDLE;
      }
      break;
    case REFLOW_STATE_MENU_TOP_I:
        myGLCD.setFont(BigFont);
        myGLCD.setColor(250, 180, 000);
        myGLCD.print("I=",230, 205);
        myGLCD.setColor(VGA_SILVER);
        myGLCD.print("P=",55, 205);
        myGLCD.setFont(SevenSegNumFont);
        myGLCD.printNumI(kp1,25, 240,3,'0');
        myGLCD.setColor(250, 180, 000);
        myGLCD.printNumI(ki1,200, 240,3,'0');
        
      if (upSwitchState == HIGH && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        ki1 = ki1 + 1;
        if (ki1 >= 500)
        {
          ki1 = 500;
        }
      }
      if (downSwitchState == HIGH && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        ki1 = ki1 - 1;
        if (ki1 <= 0)
        {
          ki1 = 0;
        }
      }
      if (okSwitchState == HIGH && !button_state && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        button_state = true;
      }
      if (okSwitchState == LOW && button_state && ( millis() - ms_button)>100)
      {        
        ms_button =  millis();
        button_state = false;
        EEPROM.write(((currentProfile - 1) * 6 + 173), ki1);
        reflowState = REFLOW_STATE_MENU_TOP_D;
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button)>60)
      {
        ms_button =  millis();
        reflowState = REFLOW_STATE_IDLE;
      }
      break;
    case REFLOW_STATE_MENU_TOP_D:
        myGLCD.setFont(BigFont);
        myGLCD.setColor(250, 180, 000);
        myGLCD.print("D=",405, 205);
        myGLCD.setColor(VGA_SILVER);
        myGLCD.print("I=",230, 205);
        myGLCD.setFont(SevenSegNumFont);
        myGLCD.printNumI(ki1,200, 240,3,'0');
        myGLCD.setColor(250, 180, 000);
        myGLCD.printNumI(kd1,375, 240,3,'0');
      if (upSwitchState == HIGH && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        kd1 = kd1 + 1;
        if (kd1 >= 500)
        {
          kd1 = 500;
        }
      }
      if (downSwitchState == HIGH && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        kd1 = kd1 - 1;
        if (kd1 <= 0)
        {
          kd1 = 0;
        }
      }
      if (okSwitchState == HIGH && !button_state && ( millis() - ms_button)>100)
      {
        ms_button =  millis();
        button_state = true;
      }
      if (okSwitchState == LOW && button_state && ( millis() - ms_button)>100)
      {        
        ms_button =  millis();
        button_state = false;
        EEPROM.write(((currentProfile - 1) * 6 + 174), kd1);        
        myGLCD.clrScr();
        reflowState = REFLOW_STATE_IDLE;
      }
      if (cancelSwitchState == HIGH && ( millis() - ms_button)>60)
      {
        ms_button =  millis();
        myGLCD.clrScr();
        reflowState = REFLOW_STATE_IDLE;
      }
      break;
      
//"Старт и процесс пайки", рост температуры с заданной скоростью
    case REFLOW_STATE_STEP_RAMP:
      if (updateScreen) {
        myGLCD.setFont(BigFontRus);
        myGLCD.setColor(VGA_RED);
        myGLCD.textRus("СТАРТ",370, 5);
        myGLCD.textRus("ПРОФИЛЬ",0, 5);
        myGLCD.setColor(VGA_LIME);
        myGLCD.textRus("ШАГ",5, 300);
        myGLCD.textRus("ВРЕМЯ",100, 300);
        myGLCD.textRus("СКОРОСТЬ",280, 300);
        myGLCD.setColor(250, 180, 000);
        myGLCD.textRus("УСТАН",5, 153);
        myGLCD.textRus("УСТАН",5, 266);
        myGLCD.textRus("НАГРЕВ",235, 153);
        myGLCD.textRus("НАГРЕВ",235, 266);
        myGLCD.setFont(SevenSegNumFont);
        myGLCD.printNumI(temperatureStep[editStep],90, 120,3,'0');
        myGLCD.printNumI(SP2,90, 232,3,'0');
        updateScreen = false;
      }
        if (profileName == 1){
        myGLCD.setFont(BigFontRus);
        myGLCD.setColor(VGA_RED);
        myGLCD.textRus("СНЯТИЕ ЧИПА БЕССВИНЕЦ",75, 40);
        }
        if (profileName == 2){
        myGLCD.setFont(BigFontRus);
        myGLCD.setColor(VGA_RED);
        myGLCD.textRus(" СНЯТИЕ ЧИПА СВИНЕЦ   ",75, 40);
        }
        if (profileName == 3){
        myGLCD.setFont(BigFontRus);
        myGLCD.setColor(VGA_RED);
        myGLCD.textRus("УС-ВКА ЧИПА БЕССВИНЕЦ",75, 40);
        }
        if (profileName == 4){
        myGLCD.setFont(BigFontRus);
        myGLCD.setColor(VGA_RED);
        myGLCD.textRus(" УС-ВКА ЧИПА СВИНЕЦ   ",75, 40);
        }
        if (profileName == 5){
        myGLCD.setFont(BigFontRus);
        myGLCD.setColor(VGA_RED);
        myGLCD.textRus("   НИЖНИЙ ПОДОГРЕВ     ",75, 40);
      }

//фиксируем размер стола
    case REFLOW_STATE_TABLE_SIZE:
      if (tableSize == 1) {
       digitalWrite(P1_PIN, HIGH);
       }
      if (tableSize == 2) {
       digitalWrite(P1_PIN, HIGH);
       digitalWrite(P2_PIN, HIGH);
       }
      if (tableSize == 3) {
       digitalWrite(P1_PIN, HIGH);
       digitalWrite(P2_PIN, HIGH);
       digitalWrite(P3_PIN, HIGH);
       }
       
      if (tc1 >= 116 && !TopStart) TopStart = true;  //если температура низа вышла на уставку включаем верхний нагреватель
      
      if (TopStart == true){   // включен верхний нагреватель      
      if (flag == 0)           //фиксируем стартовую температуру
       {
        startTemp = tc1;
        flag = 1;
       }
//устанавливаем нужный шаг, до которой нагрета плата      
      if (startTemp > temperatureStep[currentStep -1]){
        for (x=1; startTemp > temperatureStep[currentStep - 1]; currentStep++){
          x++;
       }
       }      
      if (currentStep > x && flag == 1){
        flag = 0;
        startTemp = temperatureStep[currentStep - 2];        
        flag = 1;
       }
        myGLCD.setFont(BigFont);
        myGLCD.setColor(250, 180, 000);
        myGLCD.printNumI(currentStep,60, 300);
        myGLCD.setFont(SevenSegNumFont);
        myGLCD.printNumI(temperatureStep[currentStep - 1],90, 120);
      
//счётчик скорости роста температуры             
      if ((currentMillis - previousMillis) > 1000 * rampRateStep[currentStep - 1] * 0.1) //скорость роста температуры от 0.1с. до 3с.
       {
        previousMillis = currentMillis;
        counter = counter + 1;
        setpointRamp = counter + startTemp;
        myGLCD.setFont(BigFont);
        myGLCD.setColor(250, 180, 000);
        myGLCD.printNumI(setpointRamp,420, 300,3,'0');
        Setpoint1 = setpointRamp;
       }
       } //закрывам скобку
 
      if (setpointRamp >= temperatureStep[currentStep - 1]) //если достигли нужной температуры
       {
        myGLCD.setColor(VGA_RED);
        myGLCD.printNumI(temperatureStep[currentStep - 1],420, 300);
        reflowState = REFLOW_STATE_STEP;
       }
      if (cancelSwitchState == HIGH && ( millis() - ms_button)>50)
       {
        ms_button =  millis();
        digitalWrite(P1_PIN, LOW);
        digitalWrite(P2_PIN, LOW);
        digitalWrite(P3_PIN, LOW);
        digitalWrite(P4_PIN, LOW);
        
        reflowStatus = REFLOW_STATUS_OFF;
        reflowState = REFLOW_STATE_IDLE;
        updateScreen = true;
       }
      break;
      
    case REFLOW_STATE_STEP:
      Setpoint1 = temperatureStep[currentStep - 1];
      if (Input1 >= temperatureStep[currentStep - 1])
       {
        counter = 0;
        reflowState = REFLOW_STATE_STEP_DWELL;        
       }
      if (cancelSwitchState == HIGH && ( millis() - ms_button)>60)
       {
        ms_button =  millis();
        digitalWrite(P1_PIN, LOW);
        digitalWrite(P2_PIN, LOW);
        digitalWrite(P3_PIN, LOW);
        digitalWrite(P4_PIN, LOW);
        updateScreen = true;
        
        reflowStatus = REFLOW_STATUS_OFF;
        reflowState = REFLOW_STATE_IDLE;
       }
      break;
//считаем время перехода на следующий шаг
    case REFLOW_STATE_STEP_DWELL:
      if (currentMillis - previousMillis > 1000) 
       {
        previousMillis = currentMillis;
        counter = counter + 1;
        TimerStep = counter;
        myGLCD.setFont(BigFont);
        myGLCD.setColor(250, 180, 000);
        myGLCD.printNumI(TimerStep,190, 300,2,'0');
       }
      if (counter == dwellTimerStep[currentStep - 1]) //если счётчик равен установленному времени
       {
        myGLCD.setFont(BigFont);
        myGLCD.setColor(VGA_RED);
        myGLCD.printNumI(TimerStep,190, 300,2,'0');
        tone(buzzerPin, 1045, 500);  //звуковой сигнал
        counter = 0;
        setpointRamp = 0;
        
      if (profileSteps == currentStep) //если достигли последнего шага 
       {
          currentStep = 1;
          x = 1;      //устанавливаем переменную в исходное состояние
          flag = 0;   //после завершения профиля сбрасываем флаг
          reflowState = REFLOW_STATE_COMPLETE;
       }
      else //если шаг не последний
       {          
          currentStep++; //переходим на следующий шаг
          reflowState = REFLOW_STATE_STEP_RAMP;          
       }
        }
      if (cancelSwitchState == HIGH && ( millis() - ms_button)>60)
       {
        ms_button =  millis();
        digitalWrite(P1_PIN, LOW);
        digitalWrite(P2_PIN, LOW);
        digitalWrite(P3_PIN, LOW);
        digitalWrite(P4_PIN, LOW);
        updateScreen = true;
        
        reflowStatus = REFLOW_STATUS_OFF;
        reflowState = REFLOW_STATE_IDLE;
       }
      break;
//завершение пайки      
    case REFLOW_STATE_COMPLETE:
       digitalWrite(P1_PIN, LOW);
       digitalWrite(P2_PIN, LOW);
       digitalWrite(P3_PIN, LOW);
       digitalWrite(P4_PIN, LOW);
                  
      reflowStatus = REFLOW_STATUS_OFF;
      reflowState = REFLOW_STATE_IDLE;      
      updateScreen = true;
      break;
     }
//включение нагревателей
    if (reflowStatus == REFLOW_STATUS_ON)
     {
    if (millis() > nextRead1)
     {
      Input1 = filter1(thermocouple1.readCelsius());
      Input2 = filter2(thermocouple2.readCelsius());
      tc1 = Input1;
      tc2 = Input2;

      sprintf (buf, "OK%03d%03d%03d%03d\r\n", int(Output1), int(Output2), tc1, tc2);
      Serial.print(buf);
      
      nextRead1 += SENSOR_SAMPLING_TIME;
    
    if (isnan(Input1)) {
       myGLCD.setFont(BigFont);
       myGLCD.setColor(VGA_RED);
       myGLCD.print("ERORR",360, 140);
     } else {
       myGLCD.setFont(SevenSegNumFont);
       myGLCD.setColor(VGA_RED);
       myGLCD.printNumI(tc1,345, 120,3,'0');
     }
    if (isnan(Input2)) {
       myGLCD.setFont(BigFont);
       myGLCD.setColor(VGA_RED);
       myGLCD.print("ERORR",360, 250);
     } else {
       myGLCD.setFont(SevenSegNumFont);
       myGLCD.setColor(VGA_RED);
       myGLCD.printNumI(tc2,345, 232,3,'0');
     }
     } 
      myPID1.SetTunings(kp1, ki1, kd1);
      myPID2.SetTunings(kp2, ki2, kd2);
      myPID1.SetOutputLimits(min_pwr_TOPStep[currentStep - 1], max_pwr_TOPStep[currentStep - 1]);        //установка минимального и максимального значения (от 0 до 100)
      myPID2.SetOutputLimits(min_pwr_BOTTOM, max_pwr_BOTTOM);                                            //которое будет передано на управление диммером
      myPID1.Compute();
      myPID2.Compute();
      attachInterrupt(0, Dimming, RISING); // настроить порт прерывания(0 или 1) 2й или 3й цифровой пин

      if (tc1 == 115){
       digitalWrite(P4_PIN, HIGH);
       tone(buzzerPin, 1045, 500);  //звуковой сигнал
       } 
      if (tc1 >= 250)
       {
       digitalWrite(P1_PIN, LOW);
       digitalWrite(P2_PIN, LOW);
       digitalWrite(P3_PIN, LOW);
       digitalWrite(P4_PIN, LOW);
       tone(buzzerPin, 1045, 400);  //звуковой сигнал
       
       reflowStatus = REFLOW_STATUS_OFF;
       reflowState = REFLOW_STATE_IDLE;    
       updateScreen = true;
       }       
      if (!TopStart) Output1=0;    
       }
      else
       {
       digitalWrite(RelayPin1, LOW);
       digitalWrite(RelayPin2, LOW);
       }
       }
      
void Dimming()
{
  OutPWR_TOP();
  OutPWR_BOTTOM();
}

void OutPWR_TOP(){
       reg1 = Output1 + er1; //pwr- задание выходной мощности в %,в текущем шаге профиля, er- ошибка округления   
        if (reg1 < 50){
          out1 = LOW;
          er1 = reg1; // reg- переменная для расчетов
          }
        else {
          out1 = HIGH;
          er1 = reg1-100;
        }
       digitalWrite(RelayPin1,out1);//пин через который осуществляется дискретное управление   
       }
       
void OutPWR_BOTTOM(){
       reg2 = Output2 + er2; //pwr- задание выходной мощности в %, er- ошибка округления
        if (reg2 < 50){
          out2 = LOW;
          er2 = reg2; // reg- переменная для расчетов
        }
        else {
          out2 = HIGH;
          er2 = reg2-100;
        }
       digitalWrite(RelayPin2,out2);//пин через который осуществляется дискретное управление
       }
       
  float filter1(float val_1) {  //функция фильтрации показений "Верхней" термопары
  Pc1 = P1 + varProcess1;
  G1 = Pc1/(Pc1 + varTerm1);
  P1 = (1-G1)*Pc1;
  Xp1 = Xe1;
  Zp1 = Xp1;
  Xe1 = G1*(val_1-Zp1)+Xp1; // "фильтрованное" значение
  return(Xe1);
  }
  float filter2(float val_2) {  //функция фильтрации показений "Нижней" термопары
  Pc2 = P2 + varProcess2;
  G2 = Pc2/(Pc2 + varTerm2);
  P2 = (1-G2)*Pc2;
  Xp2 = Xe2;
  Zp2 = Xp2;
  Xe2 = G2*(val_2-Zp2)+Xp2; // "фильтрованное" значение
  return(Xe2);
  }
